@@ -20,10 +20,8 @@ class SpectralBridges(BaseEstimator, ClusterMixin):
     """Spectral Bridges clustering algorithm.
 
     Attributes:
-        random_state (int | None): Determines random number generation for centroid
-            initialization.
         n_clusters (int): The number of clusters to form.
-        n_nodes (int | None): Number of nodes or initial clusters.
+        n_nodes (int): Number of nodes or initial clusters.
         p (int | float): Power of the alpha_i.
         n_iter (int): Number of iterations to run the k-means algorithm.
         n_local_trials (int | None): Number of seeding trials for centroids
@@ -33,15 +31,15 @@ class SpectralBridges(BaseEstimator, ClusterMixin):
         tol (float): Tolerance for the normalized eigengap.
         affinity_transform (AffinityTransform): Affinity transform to apply to the
             affinity matrix.
-        cluster_centers_ (list[np.ndarray] | None): Coordinates of cluster
-            centers.
-        labels_ (np.ndarray): Labels of each cluster (index) in X.
-        ngap_ (float): The normalized eigengap.
-        embedding_ (np.ndarray): Embedding of the data.
+        cluster_centers_ (np.ndarray | None): Coordinates of cluster centers.
+        cluster_labels_ (np.ndarray | None): Labels of each cluster.
+        labels_ (np.ndarray | None): Labels of each data point.
+        ngap_ (float | None): The normalized eigengap.
+        embedding_ (np.ndarray | None): Embedding of the data.
     """
 
     n_clusters: int
-    n_nodes: int | None
+    n_nodes: int
     p: int | float
     n_iter: int
     n_local_trials: int | None
@@ -49,7 +47,8 @@ class SpectralBridges(BaseEstimator, ClusterMixin):
     tol: float
     no_clustering: bool
     affinity_transform: AffinityTransform
-    cluster_centers_: list[np.ndarray] | None
+    cluster_centers_: np.ndarray | None
+    cluster_labels_: np.ndarray | None
     labels_: np.ndarray | None
     ngap_: float | None
     embedding_: np.ndarray | None
@@ -58,7 +57,7 @@ class SpectralBridges(BaseEstimator, ClusterMixin):
         {
             "X": ["array-like"],
             "n_clusters": [Interval(Integral, 2, None, closed="left")],
-            "n_nodes": [Interval(Integral, 2, None, closed="left"), None],
+            "n_nodes": [Interval(Integral, 2, None, closed="left")],
             "p": [Interval(Real, 0, None, closed="left")],
             "n_iter": [Interval(Integral, 1, None, closed="left")],
             "n_local_trials": [Interval(Integral, 1, None, closed="left"), None],
@@ -72,7 +71,7 @@ class SpectralBridges(BaseEstimator, ClusterMixin):
     def __init__(
         self,
         n_clusters: int,
-        n_nodes: int | None = None,
+        n_nodes: int,
         *,
         p: int | float = 2,
         n_iter: int = 20,
@@ -115,7 +114,7 @@ class SpectralBridges(BaseEstimator, ClusterMixin):
         self.ngap_ = None
         self.embedding_ = None
 
-        if self.n_nodes is not None and self.n_nodes <= self.n_clusters:
+        if self.n_nodes <= self.n_clusters:
             raise ValueError(
                 f"n_nodes must be greater than n_clusters, got {self.n_nodes} <= "
                 f"{self.n_clusters}"
@@ -144,12 +143,10 @@ class SpectralBridges(BaseEstimator, ClusterMixin):
         """
         X = np.asarray(X)  # type: ignore
 
-        if X.shape[0] < self.n_clusters:
+        if X.shape[0] < self.n_nodes:
             raise ValueError(
-                f"n_samples={X.shape[0]} must be >= n_clusters={self.n_clusters}."
+                f"n_samples={X.shape[0]} must be >= n_nodes={self.n_nodes}."
             )
-        if self.n_nodes is None:
-            raise ValueError("n_nodes must be provided")
 
         kmeans = KMeans(
             self.n_nodes,
@@ -158,13 +155,13 @@ class SpectralBridges(BaseEstimator, ClusterMixin):
             self.random_state,
         )
         kmeans.fit(X)
-        centers = cast(np.ndarray, kmeans.cluster_centers_)
+        self.cluster_centers_ = cast(np.ndarray, kmeans.cluster_centers_)
 
         affinity = np.empty((self.n_nodes, self.n_nodes), dtype=np.float64)
 
         X_centered = [
             np.array(
-                X[kmeans.labels_ == i] - cast(np.ndarray, kmeans.cluster_centers_)[i],
+                X[kmeans.labels_ == i] - self.cluster_centers_[i],
                 dtype=np.float64,
                 order="F",
             )
@@ -175,7 +172,9 @@ class SpectralBridges(BaseEstimator, ClusterMixin):
         counts = counts[None, :] + counts[:, None]
 
         for i in range(self.n_nodes):
-            segments = np.asfortranarray(centers - centers[i])
+            segments = np.asfortranarray(
+                self.cluster_centers_ - self.cluster_centers_[i]
+            )
             dists = np.einsum("ij,ij->i", segments, segments)
             dists[i] = 1
 
@@ -208,14 +207,13 @@ class SpectralBridges(BaseEstimator, ClusterMixin):
         if self.no_clustering:
             return self
 
-        labels = cast(
+        self.cluster_labels_ = cast(
             np.ndarray,
             KMeans(self.n_clusters, self.n_iter, self.n_local_trials, self.random_state)
             .fit(self.embedding_)  # type: ignore
             .labels_,
         )
-        self.cluster_centers_ = [centers[labels == i] for i in range(self.n_clusters)]
-        self.labels_ = labels[kmeans.labels_]
+        self.labels_ = self.cluster_labels_[kmeans.labels_]
 
         return self
 
@@ -239,7 +237,7 @@ class SpectralBridges(BaseEstimator, ClusterMixin):
         Returns:
             np.ndarray The predicted cluster indices.
         """
-        check_is_fitted(self)
+        check_is_fitted(self, ("cluster_centers_", "cluster_labels_"))
 
         X_f32 = np.asarray(X).astype(np.float32)
 
@@ -247,16 +245,10 @@ class SpectralBridges(BaseEstimator, ClusterMixin):
             raise ValueError("X must not contain inf values")
         if np.isnan(X_f32).any():
             raise ValueError("X must not contain NaN values")
-        if self.cluster_centers_ is None:
-            raise ValueError("SpectralBridges must be fitted first")
-
-        cluster_centers = np.vstack(self.cluster_centers_)
-        cluster_cutoffs = np.cumsum(
-            [cluster.shape[0] for cluster in self.cluster_centers_]
-        )
 
         index = faiss.IndexFlatL2(X_f32.shape[1])
-        index.add(cluster_centers)  # type: ignore
-        winners = cast(np.ndarray, index.search(X_f32, 1)[1]).ravel()  # type: ignore
+        index.add(self.cluster_centers_)  # type: ignore
 
-        return np.searchsorted(cluster_cutoffs, winners, side="right")
+        return cast(np.ndarray, self.cluster_labels_)[
+            cast(np.ndarray, index.search(X_f32, 1)[1]).ravel()  # type: ignore
+        ]
